/*
 *      Program do przegladania logow z ekg.
 *      Komenda - klasa bazowa.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __COMMAND_BASE_HH__
#define __COMMAND_BASE_HH__

#include <string>
#include <vector>
#include "winmain.h"
#include "winlist.h"
#include "cmdline.h"
#include "winstatus.h"
#include "texts.h"

class command_base_t {
	// potrzebne, zeby mozna bylo poustawiac wskazniki na obiekty
	// interfejsu
	friend class ui_t;

public:
	/// Maksymalna dlugosc ciagu komendy.
	static const unsigned int max_size;
	
	command_base_t(const std::string & name, const std::string & hs = "",
			const std::string & hl = "")
			: _name(name), _help_short(hs), _help_long(hl) {}
	virtual ~command_base_t() {}

	/// Puszczenie komendy w ruch.
	virtual void run(const std::vector<std::string> & argv)=0;
	/// nazwa komendy
	const std::string & name() { return _name; }
	/// Pierwszy poziom dopelniania - czyli nazwa komendy.
	virtual const std::string & completition_lvl1() { return _name; }
	/// drugi poziom dopelniania: mozliwe argumenty komendy
	virtual const std::vector<std::string> & completition_lvl2()
	{	return _completition;	}
	/// pomoc dla komendy - jednolinijkowa
	virtual const std::string & help_short()
	{	return _help_short;	}
	/// dluga pomoc dla komendy
	virtual const std::string & help_long()
	{	return _help_long;	}
	/// Czy baza sie zmienila?
	static bool db_changed() { return _db_changed; }


	/// laduje ponownie liste chatow
	void reload_chats()
	{
		_chats->clear();
		if (_db->size() == 0)
			return;

		UinLog log = _db->log(_nicks->get_chosen_value())->second;

		if (log.statusBegin() != log.statusEnd())
			_chats->append(CHATLIST_STATUSES, 0);

		chat_reverse_iterator i=log.rbegin();
		for (; i!=log.rend(); ++i) {
			if (i->second.begin() == i->second.end())
				continue;

			_chats->append(format_date(i->first), i->first);
		}
	}


protected:
	/// ustawia listy w oknach
	void set_lists();

	/// nazwa komendy - nie moe przekroczy max_size znakw
	std::string _name;
	/// wektor dopelniania
	std::vector<std::string> _completition;
	/// stringi z trescia pomocy
	std::string _help_short, _help_long;


	/**
	 * @brief Tedy komendy moga odwolywac sie do calego programu.
	 *
	 * Trzeba zapewnic poprawnosc tych referencji tak, aby komendy,
	 * ktore beda dziedziczyly z tej klasy mogly odwolywac sie
	 * bezposrednio do potrzebnych im obiektow calego programu.
	 */

	static winmain_t * _main, * _info;
	static winlist_t * _nicks, * _chats;
	static cmdline_t * _cmdline;
	static winstatus_t * _status;
	static std::map<std::string, command_base_t*> * _cmds;
	static bool * _end_program;
	static window_t * _active_window;
	static EkgLog::dBase * _db;
	static ui_t * _ui;
	static bool _db_changed;
};


#endif /* __COMMAND_BASE_HH__ */
