/*
 *      Program do przegladania logow z ekg.
 *	Usuwanie logow dla jednego kontaktu.
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __CMD_DELLOG_HH__
#define __CMD_DELLOG_HH__


#include "command-base.h"
#include "texts.h"

/**
 * @brief Komenda quit
 */
class cmd_dellog_t : public command_base_t {
public:

	cmd_dellog_t() :  command_base_t("dellog", HELP_DELLOG_SHORT,
				HELP_DELLOG_LONG),
			_done(false)
	{}

	// ta metode trzeba wywolac po zapelnieniu listy nickow
	// i przy kazdej zmianie jej zawartosci
	void complete()
	{
		_completition.clear();

		for (winlist_t::const_str_iterator i=_nicks->str_begin();
				i!=_nicks->str_end(); ++i)
			_completition.push_back(i->str());
	}

	void run(const std::vector<std::string> & argv)
	{
		if (_db->size() == 0)
			return;

		unsigned int uin = 0;
		std::string nick;

		if (argv.size()) {
			// szukamy, czy taki log istnieje
			winlist_t::const_str_iterator si = _nicks->str_find(argv[0]);
			if (si != _nicks->str_end()) {
				uin = *(_nicks->begin()+(si-_nicks->str_begin()));
			}
			nick = argv[0];
		} else {
			uin = _nicks->get_chosen_value();
			nick = _nicks->get_chosen_str();
		}

		_db->dellog(uin);
		set_lists();
		_info->print(DELLOG_DELETED_P1+nick+DELLOG_DELETED_P2+text_cast(uin));
		_db_changed = true;
	} // run()


private:

	bool _done;
};

#endif /* __CMD_DELLOG_HH__ */
