/*
 *      Program do przegladania logow z ekg.
 *      Srodowisko wykonywania programu.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "environment.h"

#include <cstring>
#include <cstdlib>

std::string environment::_error;

environment env;

environment::environment() : _flags(EUNDEF)
{
	const char *home = getenv("HOME");
	if(!home) {
		_flags = EHOME;
		return;
	}

	std::string path = home;
	path += "/.gg/userlist";

	std::ifstream in(path.c_str());
	if(!in)
		return;

	std::string tmp, nick, uins;
	std::string::size_type pos = 0, start;
	uin_t uin;
	while (true) {
		getline(in, tmp);
		if (!in)
			break;

		pos = tmp.find(';');
		for (int i=0; i<2 & pos != std::string::npos; ++i)
			pos = tmp.find(';', pos+1);
		if (pos == std::string::npos)
			return;

		start = pos+1;
		pos = tmp.find(';', pos+1);
		nick = tmp.substr(start, pos-start);

		for (int i=0; i<2; ++i)
			pos = tmp.find(';', pos+1);
		if (pos == std::string::npos)
			return;

		start = pos+1;
		pos = tmp.find(';', pos+1);
		uins = tmp.substr(start, pos-start);
		if (uins.size())
			uin = atoi(uins.c_str());

		if (uin != 0 && nick.size())
			_userlist[uin] = nick;
	}


	// wczytanie danych o polozeniu i rodzaju logow
	path = home;
	path += "/.gg/config";

	std::ifstream cfile(path.c_str());
	if (!cfile) {
		_flags |= ECONFIG;
	}


	while(cfile) {
		std::string id;
		char ch;

		while((ch = cfile.get()) != ' ')
			id += ch;

		std::string val;
		while((ch = cfile.get()) != '\n')
			val += ch;

		_var[id] = val;

		ch = cfile.get();
		if(cfile)
		    cfile.putback(ch);
	}

	if (_var.find("log_path") != _var.end()) {
		if(_var["log_path"][0] == '~')
			_var["log_path"].replace(0,1,home);
	} else {
		_flags |= ENOPATH;
		return;
	}

	if (_var.find("log") == _var.end()) {
		_flags |= ENOLOG;
		return;
	}

	if (_var.find("uin") == _var.end()) {
		_flags |= ENOUIN;
		return;
	}

	// teraz ustawimy lokalnego nicka
	_uin = atoi(_var["uin"].c_str());
	if (errno != 0 && _userlist.find(_uin) != _userlist.end())
		_nick = _userlist[_uin];
	else {
		char * logname = getenv("LOGNAME");
		_nick = (logname != 0) ? logname : _var["uin"];
	}

	_flags = EOK;
}


const userlist_t & environment::get_userlist()
{
	return _userlist;
}

const std::string & environment::get_variable(const std::string & var) const
{
	std::map<std::string, std::string>::const_iterator i = _var.find(var);
	if (i != _var.end())
		return i->second;

	return _error;
}
