/*
 *      Program do przegladania logow z ekg.
 *      Srodowisko wykonywania programu.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __ENVIRONMENT_H__
#define __ENVIRONMENT_H__

#include <string>
#include <map>
#include <fstream>
#include "ekglog.hpp"

using namespace EkgLog;


/**
 * Informacje o srodowisku dzialania programu, wczytanie listy uinow.
 */
class environment {
public:

	environment();

	/// Zwraca mape uinow.
	const userlist_t & get_userlist();
	/// Czy dane o srodowisku sa poprawne.
	operator bool () const { return _flags == EOK; }
	/// Zwraca wartosc zmiennej.
	const std::string & get_variable(const std::string & var) const;
	/// Zwraca lokalnego nicka
	const std::string & nick() { return _nick; }
	/// Zwraca lokalny uin
	unsigned int uin() { return _uin; }

private:

	/// Zwracane przez get_variable() w przypadku pytania o nieistniejaca zmienna.
	static std::string _error;

	/// Mozliwe stany obiektu.
	enum flags_t { EUNDEF=0, EOK=1, EHOME=2, ECONFIG=4, ENOLOG=8, ENOPATH=16, ENOUIN=32 };

	/// Stan obiektu.
	unsigned int _flags;
	/// Mapa uinow na nicki.
	userlist_t _userlist;
	/// Zmienne z pliku config
	std::map<std::string,std::string> _var;
	/// lokalny nick
	std::string _nick;
	/// Lokalny uin
	unsigned int _uin;
};

extern environment env;

#endif /* __ENVIRONMENT_H__ */
