#include <locale>
#include <sstream>
#include <fstream>
#include <utility>
#include <sys/time.h>

#ifdef TEST
#include <iostream>
#endif

#include "ekglog.hpp"
#include "zfstream.h"
#include "directory.hpp"
using namespace EkgLog;
using namespace std;

// funkcja pomocnicza do generowania wyjtkw o bdach formatu logw
static inline void
Error (unsigned int line_no, const string& file) throw (runtime_error)
{
  ostringstream error_str;
  error_str << "Bd formatu logw w linii " << line_no << " w pliku " << file << ends;
  throw runtime_error (error_str.str());
}

static inline void
CheckIndex (string::size_type& idx, unsigned int line_no, const string& file) 
  throw(runtime_error)
{
  if (idx == string::npos)
    Error(line_no, file);
}

static string
ParseString (const string& str, const string::size_type beg, const string::size_type end)
{
  /* beg - indeks pocztkowy, end - indeks za-kocowy
   parsujemy lini:
     \r (dwa znaki) -> \r (jeden znak)
     \n -> \n (jak wyej)
     \' -> '
     \" -> "
     \\ -> \  */
  string ret;
  for (register string::size_type i = beg,j = beg; j<end;) {
    while ((j<end) && (str[j] != '\\'))
      ++j;
    ret.append (str, i, j-i);
    if (j == end)
      break;
    if (j == end - 1) {
      ret.append (1, '\\');
      break;
    }
    switch (str[j+1]) {
      case 'r': // omi \r
	break;
      case 'n': // nowa linia
	ret.append (1, '\n'); break;
      case '\'': // hmm - on escapuje apostrofy?
	ret.append (1, '\''); break;
      case '"':
	ret.append (1, '"'); break;
      case '\\':
	ret.append (1, '\\'); break;
      default:
	ret.append (1, '\\');
	ret.append (1, str[j+1]);
	break;
    }
    j += 2;
    i = j;
  }
  return ret;
}

static string
UnParseString (const string& str, const string::size_type beg, const string::size_type end)
{
  // odparsowujemy lini na uytek zapisu logw. Dla spjnoci interfejsu sygnatura zawiera
  // pocztek i koniec (indeks pocztkowy i za-kocowy), cho obecnie funkcja save nie
  // skorzysta na tym (waciwie to writeMsg i writeStatus).
  //
  // Escapujemy znaki ', ", \, \n (ten ostatni zamieniamy zgodnie z konwencj ekg na \r\n)
  
  string ret;
  for (register string::size_type i = beg,j = beg; j<end;) {
    while ( (j<end) && (str[j] != '\'') && (str[j] != '"') && (str[j] != '\\') && (str[j] != '\n') )
      ++j;
    ret.append (str, i, j-i);
    if (j == end)
      break;
    ret.append (1, '\\'); // escape character
    if (str[j] == '\n')
      ret.append ("r\\n");
    else
      ret.append (1, str[j]);
    i = ++j;
  }
  return ret;
}
    

static inline void
LoadFile (map<const uin_t, UinLog>& uins, const string& file, unsigned int& msg_no, unsigned int& stat_no, const dBase::log_format_t format = dBase::FILE, const uin_t uin = 0)
  throw(runtime_error)
{
  gzifstream input (file.c_str()); // plik
  if (input.fail())
    throw runtime_error ("Nie udao si otworzy pliku z logiem!");
  char buf[8192]; // bufor na lini
  unsigned int line_no = 0; // do informacji o bdach
  Message msg;
  Status stat;

  // wykorzystywane do przetwarzania wartoci numerycznych.
  istringstream i;
  uin_t cur_uin;
  log_iterator uin_iter;
  pair<log_iterator, log_iterator> search_res;
  time_t time;

  if ((format == dBase::DIR) || (format == dBase::DIRGZ)) {
    cur_uin = uin;
    uin_iter = uins.insert (make_pair(cur_uin, UinLog(cur_uin))).first;
  }
  
  while (input.peek() != gzifstream::traits_type::eof()) {
    // wczytujemy lini do bufora - to chyba najszybszy sposb ze strumieniami
    input.getline (buf, 8192);
    string line (buf);
    while (!input.eof() && input.fail()) {
      // jest co jeszcze do danej linii
      input.clear();
      input.getline (buf, 8192);
      line += buf;
    }

    ++line_no;

    // --- linia jest - teraz PRZETWARZANIE ---
    // odczytaj typ wpisu
    string::size_type idx = line.find_first_of (",");
    if (line[0] != 's') {
      // --- wiadomo ---
      CheckIndex(idx, line_no, file);
      string tmp (line, idx-4, 4);
      if (tmp == "send")
	msg.type (MSG_SENT);
      else
	msg.type (MSG_RECV);
      // odczytaj uin
      string::size_type idx2 = line.find_first_of (",", ++idx);
      CheckIndex(idx2, line_no, file);
      if ((format == dBase::FILE) || (format == dBase::FILEGZ)) {
	i.str (string (line, idx, idx2-idx));
	i >> cur_uin;
	i.str (""); i.clear();
	if (i.fail())
	  Error(line_no, file);
	// czy mamy taki uin w logach?
	search_res = uins.equal_range(cur_uin);
	if (search_res.first == search_res.second) // ups, nie ma jeszcze takiego
	  uin_iter = uins.insert (make_pair(cur_uin, UinLog(cur_uin))).first;
	else
	  uin_iter = search_res.first;
      }
      idx = idx2 + 1;
      // pomi nick
      idx = line.find_first_of (",", idx);
      CheckIndex(idx, line_no, file);
      // wczytaj pierwszy czas
      idx2 = line.find_first_of (",", ++idx);
      CheckIndex(idx2, line_no, file);
      i.str (string (line, idx, idx2-idx));
      i >> time;
      i.str (""); i.clear();
      if (i.fail()) // bd podczas analizy
	Error(line_no, file);
      if (msg.type() == MSG_SENT)
	msg.sent (time);
      // wczytaj czas wysania, jeli jest to wiadomo odebrana
      else if (msg.type() == MSG_RECV) {
	msg.recv (time);
	idx = idx2 + 1;
	idx2 = line.find_first_of (",", idx);
	CheckIndex(idx2, line_no, file);
	i.str (string (line, idx, idx2-idx));
	i >> time;
	i.str (""); i.clear();
	if (i.fail()) // bd podczas analizy
	  Error(line_no, file);
	msg.sent (time);
      }
      // idx2 = ostatni przecinek. po nim ju jest tylko wiadomo.
      // stary format logw - wiadomo w cudzysowach lub nie, w wielu liniach
      // nowy format logw - wiadomo w cudzysowach lub nie, w jednej linii.

      ++idx2; // pierwszy znak wiadomoci

      // Docz biec lini do treci wiadomoci
      if (line[idx2] == '"')
	// pomi cudzysowy
	msg.msg (ParseString(line, idx2+1, line.size()-1));
      else
	// nie parsuj jak nie ma cudzyslowia - nie ma czego
	msg.msg (line, idx2, line.size());

      // TODO: stary format logw - sprawd nastpn lini...
      uin_iter->second.addMessage (msg);
      ++msg_no;
    } // if wiadomo
    else {
      // --- status ---
      // odczytaj uin
      string::size_type idx2 = line.find_first_of (",", ++idx);
      CheckIndex(idx2, line_no, file);
      if ((format == dBase::FILE) || (format == dBase::FILEGZ)) {
	i.str (string (line, idx, idx2-idx));
	i >> cur_uin;
	i.str (""); i.clear();
	if (i.fail())
	  Error(line_no, file);
	// czy mamy taki uin w logach?
	search_res = uins.equal_range(cur_uin);
	if (search_res.first == search_res.second) // ups, nie ma jeszcze takiego
	  uin_iter = uins.insert (make_pair(cur_uin, UinLog(cur_uin))).first;
	else
	  uin_iter = search_res.first;
      }
      idx = idx2 + 1;
      // pomi nick
      idx = line.find_first_of (",", idx);
      CheckIndex(idx, line_no, file);
      // pomi IP
      idx = line.find_first_of (",", ++idx);
      CheckIndex(idx, line_no, file);
      // odczytaj czas
      idx2 = line.find_first_of (",", ++idx);
      CheckIndex(idx2, line_no, file);
      i.str (string (line, idx, idx2-idx));
      i >> time;
      i.str (""); i.clear();
      if (i.fail()) // bd
	Error(line_no, file);
      stat.time (time);
      // typ statusu
      idx = idx2+1;
      idx2 = line.find_first_of (",", idx);
      string stype;
      if (idx2 == string::npos)
	stype.assign (line, idx, line.size()-idx);
      else
	stype.assign (line, idx, idx2 - idx);
      
      if (stype == "notavail")
	stat.type (NOTAVAILABLE);
      else if (stype == "busy")
	stat.type (BUSY);
      else if (stype == "avail")
	stat.type (AVAILABLE);
      else
	stat.type (INVISIBLE);

      if (idx2 != string::npos) {
	// opis statusu.
	idx = idx2+1;
	if (line[idx] == '"')
	  // pomi cudzysowy
	  stat.desc (ParseString(line, idx+1, line.size()-1));
	else
	  // nie parsuj jak nie ma cudzyslowa - nie ma czego
	  stat.desc (line, idx, line.size());
      }
      else
	stat.desc ("");
      uin_iter->second.addStatus (stat);
      ++stat_no;
    } // if status

  } // while jest co czyta
  input.close();

} // LoadFile

void
dBase::open(const string& _path, const dBase::log_format_t _format,
    const time_t interval) throw(runtime_error)
{
  if (!_logs.empty()) {
    _logs.clear();
    _messages = _statuses = 0;
    _time = 0;
    _badbit = true;
  }

  string path = _path;
  dBase::log_format_t format = _format;
  userlist_t::const_iterator nickname;
  
  // Mierzymy czas
  struct timeval start, finish;
  gettimeofday (&start, NULL);
  
  if ((format == DIR) || (format == DIRGZ)) {
    Directory dir (path);
    if (dir.isbad())
      throw runtime_error (dir.get_error().c_str());
    
    string filename = dir.read_entry();
    while (filename.size()) {
      uin_t uin;
      istringstream i (filename);
      i >> uin;
      if (!i.fail()) {
	LoadFile (_logs, path + "/" + filename, _messages, _statuses, format, uin);
      }
      filename = dir.read_entry();
    }
    dir.close();
  }
  else {
    if (format == FILEGZ) {
      path.append (".gz");
    }
    LoadFile (_logs, path, _messages, _statuses);
  }
  for (log_iterator i = _logs.begin(); i!=_logs.end(); ++i) {
    i->second.fillChats(interval);
    nickname = _nicks.find (i->second.uin());
    if (nickname != _nicks.end())
      i->second.nick(nickname->second);
  }
  _badbit = false;

  gettimeofday (&finish, NULL);
  _time = finish.tv_sec-start.tv_sec + 0.000001*(finish.tv_usec-start.tv_usec);
}

static inline void 
writeMsg (ostream& out, const Message& msg, const uin_t uin, const string& nick) 
{
  msg_type t = msg.type();
  if (t == MSG_RECV)
    out << "chatrecv,";
  else
    out << "chatsend,";
  out << uin << "," << nick << ",";
  if (t == MSG_RECV)
    out << msg.recv() << ",";
  out << msg.sent() << ",\"" << UnParseString(msg.msg(), 0, msg.msg().size()) << "\"\n";
}

static inline void
writeStatus (ostream& out, const Status& status, const uin_t uin, const string& nick)
{
  out << "status," << uin << "," << nick << ",0.0.0.0:0,";
  out << status.time() << ",";
  status_type t = status.type();
  switch (t) {
    case AVAILABLE:
      out << "avail,"; break;
    case BUSY:
      out << "busy,"; break;
    case INVISIBLE:
      out << "invisible,"; break;
    case NOTAVAILABLE:
      out << "notavail,"; break;
  }
  out << "\"" << UnParseString(status.desc(), 0, status.desc().size()) << "\"\n";
}

bool lt_logline (const pair<time_t, string>& l, const pair<time_t, string>& r) {
  return (l.first < r.first);
}

void
dBase::save (const string& path, const log_format_t format) throw(runtime_error)
{
  if (_badbit)
    return;
  if ((format == DIR) || (format == DIRGZ))
    // kady uin w osobnym pliku - path jest katalogiem.
    for (const_log_iterator i = _logs.begin(); i!=_logs.end(); ++i) {
      ostringstream filename;
      filename << path << '/' << i->second.uin();
      ostream* file;
      try {
	if (format == DIRGZ) {
	  filename << ".gz";
	  file = new gzofstream (filename.str().c_str(), ios::out|ios::trunc);
	}
	else
	  file = new ofstream (filename.str().c_str(), ios::out|ios::trunc);
      }
      catch (bad_alloc) {
	throw runtime_error ("Brak pamici operacyjnej na strumie plikowy!\n");
      }
      if ( *file == false) {
	ostringstream error;
	error << "Nie mona otworzy pliku: " << filename.str() << endl;
	throw runtime_error (error.str());
      }
      const_msg_iterator msg_it = i->second.msgBegin();
      const_status_iterator stat_it = i->second.statusBegin();
      while ((msg_it != i->second.msgEnd()) && (stat_it != i->second.statusEnd())) {
	if (msg_it->recv() < stat_it->time()) {
	  writeMsg(*file, *msg_it, i->second.uin(), i->second.nick());
	  ++msg_it;
	}
	else {
	  writeStatus(*file, *stat_it, i->second.uin(), i->second.nick());
	  ++stat_it;
	}
      }
      // co zostao - trzeba to przepisa do koca. tylko jedna z poniszych ptli
      // wykona si cho raz.
      while (msg_it != i->second.msgEnd()) {
	writeMsg(*file, *msg_it, i->second.uin(), i->second.nick());
	++msg_it;
      }
      while (stat_it != i->second.statusEnd()) {
	writeStatus(*file, *stat_it, i->second.uin(), i->second.nick());
	++stat_it;
      }
      // cay uin zapisany.
      delete file;
    }
  else {
    // format == FILE || format == FILEGZ
    // Algorytm:
    //   wrzucamy wszystko do jednej listy, sortujc po czasach (osobno dla statusw
    //   i wiadomoci). Potem postpujemy jak wyej.
    
    // otwrz plik
    ostream* file;
    try {
      if (format == FILEGZ)
	file = new gzofstream( string(path + ".gz").c_str(), ios::out|ios::trunc );
      else
	file = new ofstream(path.c_str(), ios::out|ios::trunc);
    }
    catch (bad_alloc) {
      throw runtime_error ("Brak pamici operacyjnej na strumie plikowy!\n");
    }
    if ( *file == false ) {
      ostringstream error;
      error << "Nie mona otworzy pliku: " << path;
      if (format == FILEGZ)
	error << ".gz";
      error << endl;
      throw runtime_error (error.str());
    }
    
    list<pair<time_t, string> > messages, statuses;

    for (const_log_iterator i = _logs.begin(); i!=_logs.end(); ++i) {
      ostringstream log_line;
      string empty_string = "";
      for (const_msg_iterator j = i->second.msgBegin(); j!=i->second.msgEnd(); ++j) {
	writeMsg(log_line, *j, i->second.uin(), i->second.nick());
	messages.push_back( make_pair(j->recv(), log_line.str()) );
	log_line.str(empty_string);
      }
      for (const_status_iterator j = i->second.statusBegin(); j!=i->second.statusEnd(); ++j) {
	writeStatus(log_line, *j, i->second.uin(), i->second.nick());
	statuses.push_back( make_pair(j->time(), log_line.str()) );
	log_line.str(empty_string);
      }
    }

    // sortuj listy
    messages.sort(lt_logline);
    statuses.sort(lt_logline);

    // zapisz w pliku
    list<pair<time_t, string> >::iterator msg_it = messages.begin();
    list<pair<time_t, string> >::iterator stat_it = statuses.begin();
    while ((msg_it != messages.end()) && (stat_it != statuses.end())) {
      if (msg_it->first < stat_it->first) {
	(*file) << msg_it->second;
	++msg_it;
      }
      else {
	(*file) << stat_it->second;
	++stat_it;
      }
    }
    // co zostao - trzeba to przepisa do koca. tylko jedna z poniszych ptli
    // wykona si cho raz.
    while (msg_it != messages.end()) {
      (*file) << msg_it->second;
      ++msg_it;
    }
    while (stat_it != statuses.end()) {
      (*file) << stat_it->second;
      ++stat_it;
    }
    // cay uin zapisany.
    delete file;

      
  } // format == FILE(GZ)
      
} // dBase::save

#ifdef TEST
int main () {
  dBase db;
  userlist_t users;
  users[2299227] = "julia";
  try {
    db.open ("history2", dBase::DIR, users);
//    db.open ("logs", dBase::FILEGZ, users);
  }
  catch (runtime_error& e) {
    cout << "Bd!!! " << e.what() << endl;
    return 1;
  }
  cout << "Odczytano " << db.messages() << " wiadomoci i " << db.statuses() << " statusw.\n";
  cout << "Zajo to " << db.time() << " sekund.\n";
  cout << "test --- \n";

  cout << "zapisujemy z powrotem do history2...\n";
  try {
    db.save ("history3", dBase::DIR);
  }
  catch (std::runtime_error& e) {
    cerr << "Bad!!! " << e.what() << endl;
    return 2;
  }
  
}
#endif
