/*
 *      Program do przegladania logow z ekg.
 *      Klasa reprezentujaca jedna linie z okna.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "line.h"
#include "stuff.h"

#include <cstring>
#include <cassert>
#include <vector>
#include <queue>

line_t::line_t(const char * str, int fg, int bold, int bg, flags_t flags)
{
	if (flags & NO_TAGS) {
		_content = str;
		_tags.push_back(tag_t(COLOR_WHITE,0,COLOR_BLACK,strlen(str)));
		return;
	}

	/* przeszukujemy str w celu znalezienia miejsc
	 * zmiany koloru tekstu */
	tag_t tag(fg,bold,bg,0);

	while(*str) {
		switch(*str) {
			// znak \ powinien cos escape'owac; jesli jednak
			// nie escapuje niczego poprawnego, dodaje go
			// jakby byl zwyklym znakiem
			// dodatkowo zakladam, ze w wypadku poprawnego escapowania
			// nie ma blednych ciagow, czyli przy \c wystepuja jeszcze 3 cyfry
			case '\\':

				switch(*++str) {
					case 'c':
						if(tag._lenght)
							_tags.push_back(tag);

						tag._fg     = *++str-'0';
						tag._bold   = *++str-'0';
						tag._bg     = *++str-'0';
						tag._lenght = 0;
						++str;
						break;
					case 'd':
						if(tag._lenght)
							_tags.push_back(tag);

						tag(fg,bold,bg,0);
						++str;
						break;

					// jesli wystepuja znaki \\ to znaczy, ze byc moze
					// zrobiono to, zeby po znaku \ wystapily litery c lub d
					case '\\':
					case '\'':
					case '\"':
						_content += '\\';
						tag._lenght += 1;
						break;

					case 0:
						break;

					default:
						_content += '\\';
						tag._lenght += 1;
				}
				break;

			default:
				_content += *str;
				++tag._lenght;
				++str;
				break;
		}/* switch(*k) */
	}/* while() */

	if(tag._lenght)
		_tags.push_back(tag);
}


void line_t::print(WINDOW *w) const
{
	int pos=0;

	for(std::list<tag_t>::const_iterator i=_tags.begin(); i!=_tags.end(); ++i) {
		wattron(w, color_pair(i->_fg, i->_bold, i->_bg));
		waddstr(w, const_cast<char*>(_content.substr(pos, i->_lenght).c_str()));
		pos += i->_lenght;
		wattroff(w, color_pair(i->_fg, i->_bold, i->_bg));
	}
}


void line_t::print(std::ostream &o) const
{
	o << _content;
}


/*
 * Przedzial kolorowania linii - struktura pomocnicza dla metody
 * addtag().
 */
struct range {
	int begin,end,fg,bold,bg;

	range(int a, int b, int c, int d, int e)
	    :begin(a),end(b),fg(c),bold(d),bg(e) {}


	bool operator < (const range & r) const
	{
		return begin > r.begin;
	}

	operator line_t::tag_t ()
	{
		return line_t::tag_t(fg,bold,bg,end-begin);
	}
};


void line_t::addtag(const tag_t & tag, int start)
{
	std::priority_queue<range> p;
	int s=0;

	for(std::list<tag_t>::iterator i=_tags.begin(); i!=_tags.end(); ++i) {
		p.push(range(s, s+i->_lenght, i->_fg, i->_bold, i->_bg));
		s += i->_lenght;
	}

	s = 0;
	_tags.clear();

	int nb = start, ne = start+tag._lenght;

	while(p.size()) {
		range r = p.top();
		/* trzeba rozpatrzyc kazdy przypadek zawierania sie aktualnych
		 * przedzialow oraz dodawanego
		 * wszystkie przedzialy sa w postaci [ ) */
		if(nb >= r.end) {
			_tags.push_back(r);

		} else if (r.begin >= ne) {
			_tags.push_back(r);

		} else if (nb > r.begin && nb < r.end && ne >= r.end) {
			_tags.push_back(tag_t(r.fg, r.bold, r.bg, nb-r.begin));
			_tags.push_back(tag);

		} else if (r.begin >= nb && ne > r.begin && ne < r.end) {
			_tags.push_back(tag);
			_tags.push_back(tag_t(r.fg, r.bold, r.bg, r.end-ne));

		} else if (r.begin < nb && ne < r.end) {
			_tags.push_back(tag_t(r.fg, r.bold, r.bg, nb-r.begin));
			_tags.push_back(tag);
			_tags.push_back(tag_t(r.fg, r.bold, r.bg, r.end-ne));

		} else if (r.begin == nb && r.end == ne) {
			_tags.push_back(tag);

		}
		// zdejmujemy ze stosu przedzial
		p.pop();
	}
	if (!_tags.size())
		_tags.push_back(tag);
}


std::ostream & operator << (std::ostream & o, const line_t & l)
{
	l.print(o);
	return o;
}
