/*
 *      Program do przegladania logow z ekg.
 *      Glowne okno.
 *
 *
 *      copyright (c) 2003 by Lukasz Bartnik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License Version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __WINMAIN_HH__
#define __WINMAIN_HH__

extern "C" {
	#include <ncurses.h>
}

#include <map>
#include <fstream>

#include "window.h"
#include "environment.h"
#include <ekglog.hpp>
using namespace EkgLog;

/**
 * @brief Okno glowne - do wyswietlania zapisu rozmow lub komunikatow.
 *
 * Glowne okno interfejsu. Przygotowane do wyswietlania zapisu czatu
 * oraz komunikatow programu. Zawiera pola i metody przydatne przy
 * wyswietlaniu wynikow wyszukiwania.
 */
class winmain_t: public window_t {
public:
	/// pobiera tytul, wymiary okna, dlugosc historii oraz wskaznik na okno wyswietlania
	winmain_t(const char *title, 
		     int width, int height,
		     int history_lenght,
		     WINDOW *win);
	virtual ~winmain_t() {}

	/// wypisuje okno na ekranie
	virtual void print();
	/// Dodaje tekst do okna.
	void print(const std::string & str, const std::string & fmt = "")
	{
		window_t::print(str, fmt);
	}
	/// nie wiem czemu, ale nie chce sie inaczej kompilowac
	void print(const std::string & str, const std::string & fmt, bool m)
	{
		window_t::print(str, fmt, m);
	}
	/// ustawia stan wyswietlane/zakryte
	void set_displayed(bool d) { _on_screen = d; _redraw = d; }
	/// zwraca true, jesli okno jest wyswietla, false, jesli ukryte
	bool is_displayed(void) { return _on_screen; }
	/// wypisz w oknie zadany czat
	void add_chat(const EkgLog::Chat & chat, const EkgLog::UinLog & ulog);
	/// Czyszczenie okna.
	void clear();
	/// Wpisz w oknie statuy.
	void add_statuses(const EkgLog::UinLog & log);
	/// ustaw znaleziony ciag
	void winmain_t::set_found(	const EkgLog::SearchResult & sr, 
					const EkgLog::Chat & chat,
					const EkgLog::UinLog & uinlog );
	/// Zrzuca zawartosc okna do pliku.
	void dump(std::ofstream & out);


protected:

	/// ile linii ma trzymac w historii
	unsigned int _history_lenght; 
	/// czy ma prawo do wyswietlania siebie na ekranie
	bool _on_screen;
	/// numery linii rozpoczecia kolejnych wiadomosci z czatu
	std::vector<unsigned int> _msg_start;
	/// iteroatory odpowiadajace wiadomosciom z wektora _msg_start
	std::vector<msg_iterator> _msg_iter;

	/// bufor podswietlonych linii
	std::vector<line_t> _colored_line;
	std::vector<unsigned int> _colored_no;

	/// ktory chat jest aktualnie wyswietlany
	unsigned int _act_uin;
	time_t _act_chat;

	/// ile wycieto z danej linii znakow
	//std::vector<unsigned int> _cut_out;
};

#endif /* __WINMAIN_HH__ */
